CREATE TABLE WAREHOUSE (
  W_ID SMALLINT DEFAULT 0 NOT NULL,
  W_NAME VARCHAR(10) DEFAULT NULL,
  W_STREET_1 VARCHAR(20) DEFAULT NULL,
  W_STREET_2 VARCHAR(20) DEFAULT NULL,
  W_CITY VARCHAR(20) DEFAULT NULL,
  W_STATE CHAR(2) DEFAULT NULL,
  W_ZIP CHAR(9) DEFAULT NULL,
  W_TAX NUMERIC(4,4) DEFAULT NULL,
  W_YTD NUMERIC(12,2) DEFAULT NULL,
  PRIMARY KEY (W_ID)
);
CREATE TABLE DISTRICT (
  D_ID SMALLINT DEFAULT 0 NOT NULL,
  D_W_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES WAREHOUSE (W_ID),
  D_NAME VARCHAR(10) DEFAULT NULL,
  D_STREET_1 VARCHAR(20) DEFAULT NULL,
  D_STREET_2 VARCHAR(20) DEFAULT NULL,
  D_CITY VARCHAR(20) DEFAULT NULL,
  D_STATE CHAR(2) DEFAULT NULL,
  D_ZIP CHAR(9) DEFAULT NULL,
  D_TAX NUMERIC(4,4) DEFAULT NULL,
  D_YTD NUMERIC(12,2) DEFAULT NULL,
  D_NEXT_O_ID INTEGER DEFAULT NULL,
  PRIMARY KEY (D_W_ID,D_ID)
);
CREATE TABLE CUSTOMER (
  C_ID INTEGER DEFAULT 0 NOT NULL,
  C_D_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES DISTRICT(D_ID),
  C_W_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES WAREHOUSE(W_ID),
  C_LAST VARCHAR(16) DEFAULT NULL,
  C_MIDDLE CHAR(2) DEFAULT NULL,
  C_FIRST VARCHAR(16) DEFAULT NULL,
  C_STREET_1 VARCHAR(20) DEFAULT NULL,
  C_STREET_2 VARCHAR(20) DEFAULT NULL,
  C_CITY VARCHAR(20) DEFAULT NULL,
  C_STATE CHAR(2) DEFAULT NULL,
  C_ZIP CHAR(9) DEFAULT NULL,
  C_PHONE CHAR(16) DEFAULT NULL,
  C_SINCE DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
  C_CREDIT CHAR(2) DEFAULT NULL,
  C_CREDIT_LIM NUMERIC(12,2) DEFAULT NULL,
  C_DISCOUNT NUMERIC(4,4) DEFAULT NULL,
  C_BALANCE NUMERIC(12,2) DEFAULT NULL,
  C_YTD_PAYMENT NUMERIC(12,2) DEFAULT NULL,
  C_PAYMENT_CNT INTEGER DEFAULT NULL,
  C_DELIVERY_CNT INTEGER DEFAULT NULL,
  C_DATA VARCHAR(500),
  PRIMARY KEY (C_W_ID,C_D_ID,C_ID)
);
CREATE TABLE HISTORY (
  H_C_ID INTEGER DEFAULT NULL REFERENCES CUSTOMER(C_ID),
  H_C_D_ID SMALLINT DEFAULT NULL REFERENCES DISTRICT(D_ID),
  H_C_W_ID SMALLINT DEFAULT NULL REFERENCES WAREHOUSE(W_ID),
  H_D_ID SMALLINT DEFAULT NULL REFERENCES DISTRICT(D_ID),
  H_W_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES WAREHOUSE(W_ID),
  H_DATE DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
  H_AMOUNT NUMERIC(6,2) DEFAULT NULL,
  H_DATA VARCHAR(24) DEFAULT NULL
);
CREATE TABLE ORDERS (
  O_ID INTEGER DEFAULT 0 NOT NULL,
  O_C_ID INTEGER DEFAULT NULL REFERENCES CUSTOMER(C_ID),
  O_D_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES DISTRICT(D_ID),
  O_W_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES WAREHOUSE(W_ID),
  O_ENTRY_D DATETIME DEFAULT CURRENT_TIMESTAMP NOT NULL,
  O_CARRIER_ID INTEGER DEFAULT NULL,
  O_OL_CNT NUMERIC(2) DEFAULT NULL,
  O_ALL_LOCAL NUMERIC(1) DEFAULT NULL,
  PRIMARY KEY (O_W_ID,O_D_ID,O_ID)
);
CREATE TABLE NEW_ORDER (
  NO_O_ID INTEGER DEFAULT 0 NOT NULL REFERENCES ORDERS(O_ID),
  NO_D_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES DISTRICT(D_ID),
  NO_W_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES WAREHOUSE(W_ID),
  PRIMARY KEY (NO_D_ID,NO_W_ID,NO_O_ID)
);
CREATE TABLE ITEM (
  I_ID INTEGER DEFAULT 0 NOT NULL,
  I_IM_ID INTEGER DEFAULT NULL,
  I_NAME VARCHAR(24) DEFAULT NULL,
  I_PRICE NUMERIC(5,2) DEFAULT NULL,
  I_DATA VARCHAR(50) DEFAULT NULL,
  PRIMARY KEY (I_ID)
);
CREATE TABLE STOCK (
  S_I_ID INTEGER DEFAULT 0 NOT NULL REFERENCES ITEM (I_ID),
  S_W_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES WAREHOUSE (W_ID),
  S_QUANTITY NUMERIC(4) DEFAULT 0 NOT NULL,
  S_DIST_01 CHAR(24) DEFAULT NULL,
  S_DIST_02 CHAR(24) DEFAULT NULL,
  S_DIST_03 CHAR(24) DEFAULT NULL,
  S_DIST_04 CHAR(24) DEFAULT NULL,
  S_DIST_05 CHAR(24) DEFAULT NULL,
  S_DIST_06 CHAR(24) DEFAULT NULL,
  S_DIST_07 CHAR(24) DEFAULT NULL,
  S_DIST_08 CHAR(24) DEFAULT NULL,
  S_DIST_09 CHAR(24) DEFAULT NULL,
  S_DIST_10 CHAR(24) DEFAULT NULL,
  S_YTD NUMERIC(8) DEFAULT NULL,
  S_ORDER_CNT NUMERIC(4) DEFAULT NULL,
  S_REMOTE_CNT NUMERIC(4) DEFAULT NULL,
  S_DATA VARCHAR(50) DEFAULT NULL,
  PRIMARY KEY (S_W_ID,S_I_ID)
);
CREATE TABLE ORDER_LINE (
  OL_O_ID INTEGER DEFAULT 0 NOT NULL REFERENCES ORDERS(O_ID),
  OL_D_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES DISTRICT(D_ID),
  OL_W_ID SMALLINT DEFAULT 0 NOT NULL REFERENCES WAREHOUSE(W_ID),
  OL_NUMBER SMALLINT DEFAULT 0 NOT NULL,
  OL_I_ID INTEGER DEFAULT NULL REFERENCES ITEM(I_ID),
  OL_SUPPLY_W_ID SMALLINT DEFAULT NULL REFERENCES WAREHOUSE(W_ID),
  OL_DELIVERY_D DATETIME DEFAULT NULL,
  OL_QUANTITY NUMERIC(2) DEFAULT NULL,
  OL_AMOUNT NUMERIC(6,2) DEFAULT 0,
  OL_DIST_INFO CHAR(24) DEFAULT NULL,
  PRIMARY KEY (OL_W_ID,OL_D_ID,OL_O_ID,OL_NUMBER)
);